import polars as pl
from tqdm.auto import tqdm

from aiondata.datasets import CachedDataset

ZINC20_TRANCHES = [
    "BAAA",
    "BAAB",
    "BABA",
    "BABB",
    "BACA",
    "BACB",
    "BAEA",
    "BAEB",
    "CAAA",
    "CAAB",
    "CABA",
    "CABB",
    "CACA",
    "CACB",
    "CAEA",
    "CAEB",
    "DAAA",
    "DAAB",
    "DABA",
    "DABB",
    "DACA",
    "DACB",
    "DAEA",
    "DAEB",
    "EAAA",
    "EAAB",
    "EABA",
    "EABB",
    "EACA",
    "EACB",
    "EAEA",
    "EAEB",
    "FAAA",
    "FAAB",
    "FABA",
    "FABB",
    "FACA",
    "FACB",
    "FAEA",
    "FAEB",
    "GAAA",
    "GAAB",
    "GABA",
    "GABB",
    "GACA",
    "GACB",
    "GAEA",
    "GAEB",
    "HAAA",
    "HAAB",
    "HABA",
    "HABB",
    "HACA",
    "HACB",
    "HAEA",
    "HAEB",
    "IAAA",
    "IAAB",
    "IABA",
    "IABB",
    "IACA",
    "IACB",
    "IAEA",
    "IAEB",
    "JAAA",
    "JAAB",
    "JABA",
    "JABB",
    "JACA",
    "JACB",
    "JAEA",
    "JAEB",
    "BBAA",
    "BBAB",
    "BBBA",
    "BBBB",
    "BBCA",
    "BBCB",
    "BBEA",
    "BBEB",
    "CBAA",
    "CBAB",
    "CBBA",
    "CBBB",
    "CBCA",
    "CBCB",
    "CBEA",
    "CBEB",
    "DBAA",
    "DBAB",
    "DBBA",
    "DBBB",
    "DBCA",
    "DBCB",
    "DBEA",
    "DBEB",
    "EBAA",
    "EBAB",
    "EBBA",
    "EBBB",
    "EBCA",
    "EBCB",
    "EBEA",
    "EBEB",
    "FBAA",
    "FBAB",
    "FBBA",
    "FBBB",
    "FBCA",
    "FBCB",
    "FBEA",
    "FBEB",
    "GBAA",
    "GBAB",
    "GBBA",
    "GBBB",
    "GBCA",
    "GBCB",
    "GBEA",
    "GBEB",
    "HBAA",
    "HBAB",
    "HBBA",
    "HBBB",
    "HBCA",
    "HBCB",
    "HBEA",
    "HBEB",
    "IBAA",
    "IBAB",
    "IBBA",
    "IBBB",
    "IBCA",
    "IBCB",
    "IBEA",
    "IBEB",
    "JBAA",
    "JBAB",
    "JBBA",
    "JBBB",
    "JBCA",
    "JBCB",
    "JBEA",
    "JBEB",
    "BCAA",
    "BCAB",
    "BCBA",
    "BCBB",
    "BCCA",
    "BCCB",
    "BCEA",
    "BCEB",
    "CCAA",
    "CCAB",
    "CCBA",
    "CCBB",
    "CCCA",
    "CCCB",
    "CCEA",
    "CCEB",
    "DCAA",
    "DCAB",
    "DCBA",
    "DCBB",
    "DCCA",
    "DCCB",
    "DCEA",
    "DCEB",
    "ECAA",
    "ECAB",
    "ECBA",
    "ECBB",
    "ECCA",
    "ECCB",
    "ECEA",
    "ECEB",
    "FCAA",
    "FCAB",
    "FCBA",
    "FCBB",
    "FCCA",
    "FCCB",
    "FCEA",
    "FCEB",
    "GCAA",
    "GCAB",
    "GCBA",
    "GCBB",
    "GCCA",
    "GCCB",
    "GCEA",
    "GCEB",
    "HCAA",
    "HCAB",
    "HCBA",
    "HCBB",
    "HCCA",
    "HCCB",
    "HCEA",
    "HCEB",
    "ICAA",
    "ICAB",
    "ICBA",
    "ICBB",
    "ICCA",
    "ICCB",
    "ICEA",
    "ICEB",
    "JCAA",
    "JCAB",
    "JCBA",
    "JCBB",
    "JCCA",
    "JCCB",
    "JCEA",
    "JCEB",
    "BDAA",
    "BDAB",
    "BDBA",
    "BDBB",
    "BDCA",
    "BDCB",
    "BDEA",
    "BDEB",
    "CDAA",
    "CDAB",
    "CDBA",
    "CDBB",
    "CDCA",
    "CDCB",
    "CDEA",
    "CDEB",
    "DDAA",
    "DDAB",
    "DDBA",
    "DDBB",
    "DDCA",
    "DDCB",
    "DDEA",
    "DDEB",
    "EDAA",
    "EDAB",
    "EDBA",
    "EDBB",
    "EDCA",
    "EDCB",
    "EDEA",
    "EDEB",
    "FDAA",
    "FDAB",
    "FDBA",
    "FDBB",
    "FDCA",
    "FDCB",
    "FDEA",
    "FDEB",
    "GDAA",
    "GDAB",
    "GDBA",
    "GDBB",
    "GDCA",
    "GDCB",
    "GDEA",
    "GDEB",
    "HDAA",
    "HDAB",
    "HDBA",
    "HDBB",
    "HDCA",
    "HDCB",
    "HDEA",
    "HDEB",
    "IDAA",
    "IDAB",
    "IDBA",
    "IDBB",
    "IDCA",
    "IDCB",
    "IDEA",
    "IDEB",
    "JDAA",
    "JDAB",
    "JDBA",
    "JDBB",
    "JDCA",
    "JDCB",
    "JDEA",
    "JDEB",
    "BEAA",
    "BEAB",
    "BEBA",
    "BEBB",
    "BECA",
    "BECB",
    "BEEA",
    "BEEB",
    "CEAA",
    "CEAB",
    "CEBA",
    "CEBB",
    "CECA",
    "CECB",
    "CEEA",
    "CEEB",
    "DEAA",
    "DEAB",
    "DEBA",
    "DEBB",
    "DECA",
    "DECB",
    "DEEA",
    "DEEB",
    "EEAA",
    "EEAB",
    "EEBA",
    "EEBB",
    "EECA",
    "EECB",
    "EEEA",
    "EEEB",
    "FEAA",
    "FEAB",
    "FEBA",
    "FEBB",
    "FECA",
    "FECB",
    "FEEA",
    "FEEB",
    "GEAA",
    "GEAB",
    "GEBA",
    "GEBB",
    "GECA",
    "GECB",
    "GEEA",
    "GEEB",
    "HEAA",
    "HEAB",
    "HEBA",
    "HEBB",
    "HECA",
    "HECB",
    "HEEA",
    "HEEB",
    "IEAA",
    "IEAB",
    "IEBA",
    "IEBB",
    "IECA",
    "IECB",
    "IEEA",
    "IEEB",
    "JEAA",
    "JEAB",
    "JEBA",
    "JEBB",
    "JECA",
    "JECB",
    "JEEA",
    "JEEB",
    "BFAA",
    "BFAB",
    "BFBA",
    "BFBB",
    "BFCA",
    "BFCB",
    "BFEA",
    "BFEB",
    "CFAA",
    "CFAB",
    "CFBA",
    "CFBB",
    "CFCA",
    "CFCB",
    "CFEA",
    "CFEB",
    "DFAA",
    "DFAB",
    "DFBA",
    "DFBB",
    "DFCA",
    "DFCB",
    "DFEA",
    "DFEB",
    "EFAA",
    "EFAB",
    "EFBA",
    "EFBB",
    "EFCA",
    "EFCB",
    "EFEA",
    "EFEB",
    "FFAA",
    "FFAB",
    "FFBA",
    "FFBB",
    "FFCA",
    "FFCB",
    "FFEA",
    "FFEB",
    "GFAA",
    "GFAB",
    "GFBA",
    "GFBB",
    "GFCA",
    "GFCB",
    "GFEA",
    "GFEB",
    "HFAA",
    "HFAB",
    "HFBA",
    "HFBB",
    "HFCA",
    "HFCB",
    "HFEA",
    "HFEB",
    "IFAA",
    "IFAB",
    "IFBA",
    "IFBB",
    "IFCA",
    "IFCB",
    "IFEA",
    "IFEB",
    "JFAA",
    "JFAB",
    "JFBA",
    "JFBB",
    "JFCA",
    "JFCB",
    "JFEA",
    "JFEB",
    "BGAA",
    "BGAB",
    "BGBA",
    "BGBB",
    "BGCA",
    "BGCB",
    "BGEA",
    "BGEB",
    "CGAA",
    "CGAB",
    "CGBA",
    "CGBB",
    "CGCA",
    "CGCB",
    "CGEA",
    "CGEB",
    "DGAA",
    "DGAB",
    "DGBA",
    "DGBB",
    "DGCA",
    "DGCB",
    "DGEA",
    "DGEB",
    "EGAA",
    "EGAB",
    "EGBA",
    "EGBB",
    "EGCA",
    "EGCB",
    "EGEA",
    "EGEB",
    "FGAA",
    "FGAB",
    "FGBA",
    "FGBB",
    "FGCA",
    "FGCB",
    "FGEA",
    "FGEB",
    "GGAA",
    "GGAB",
    "GGBA",
    "GGBB",
    "GGCA",
    "GGCB",
    "GGEA",
    "GGEB",
    "HGAA",
    "HGAB",
    "HGBA",
    "HGBB",
    "HGCA",
    "HGCB",
    "HGEA",
    "HGEB",
    "IGAA",
    "IGAB",
    "IGBA",
    "IGBB",
    "IGCA",
    "IGCB",
    "IGEA",
    "IGEB",
    "JGAA",
    "JGAB",
    "JGBA",
    "JGBB",
    "JGCA",
    "JGCB",
    "JGEA",
    "JGEB",
    "BHAA",
    "BHAB",
    "BHBA",
    "BHBB",
    "BHCA",
    "BHCB",
    "BHEA",
    "BHEB",
    "CHAA",
    "CHAB",
    "CHBA",
    "CHBB",
    "CHCA",
    "CHCB",
    "CHEA",
    "CHEB",
    "DHAA",
    "DHAB",
    "DHBA",
    "DHBB",
    "DHCA",
    "DHCB",
    "DHEA",
    "DHEB",
    "EHAA",
    "EHAB",
    "EHBA",
    "EHBB",
    "EHCA",
    "EHCB",
    "EHEA",
    "EHEB",
    "FHAA",
    "FHAB",
    "FHBA",
    "FHBB",
    "FHCA",
    "FHCB",
    "FHEA",
    "FHEB",
    "GHAA",
    "GHAB",
    "GHBA",
    "GHBB",
    "GHCA",
    "GHCB",
    "GHEA",
    "GHEB",
    "HHAA",
    "HHAB",
    "HHBA",
    "HHBB",
    "HHCA",
    "HHCB",
    "HHEA",
    "HHEB",
    "IHAA",
    "IHAB",
    "IHBA",
    "IHBB",
    "IHCA",
    "IHCB",
    "IHEA",
    "IHEB",
    "JHAA",
    "JHAB",
    "JHBA",
    "JHBB",
    "JHCA",
    "JHCB",
    "JHEA",
    "JHEB",
    "BIAA",
    "BIAB",
    "BIBA",
    "BIBB",
    "BICA",
    "BICB",
    "BIEA",
    "BIEB",
    "CIAA",
    "CIAB",
    "CIBA",
    "CIBB",
    "CICA",
    "CICB",
    "CIEA",
    "CIEB",
    "DIAA",
    "DIAB",
    "DIBA",
    "DIBB",
    "DICA",
    "DICB",
    "DIEA",
    "DIEB",
    "EIAA",
    "EIAB",
    "EIBA",
    "EIBB",
    "EICA",
    "EICB",
    "EIEA",
    "EIEB",
    "FIAA",
    "FIAB",
    "FIBA",
    "FIBB",
    "FICA",
    "FICB",
    "FIEA",
    "FIEB",
    "GIAA",
    "GIAB",
    "GIBA",
    "GIBB",
    "GICA",
    "GICB",
    "GIEA",
    "GIEB",
    "HIAA",
    "HIAB",
    "HIBA",
    "HIBB",
    "HICA",
    "HICB",
    "HIEA",
    "HIEB",
    "IIAA",
    "IIAB",
    "IIBA",
    "IIBB",
    "IICA",
    "IICB",
    "IIEA",
    "IIEB",
    "JIAA",
    "JIAB",
    "JIBA",
    "JIBB",
    "JICA",
    "JICB",
    "JIEA",
    "JIEB",
    "BJAA",
    "BJAB",
    "BJBA",
    "BJBB",
    "BJCA",
    "BJCB",
    "BJEA",
    "BJEB",
    "CJAA",
    "CJAB",
    "CJBA",
    "CJBB",
    "CJCA",
    "CJCB",
    "CJEA",
    "CJEB",
    "DJAA",
    "DJAB",
    "DJBA",
    "DJBB",
    "DJCA",
    "DJCB",
    "DJEA",
    "DJEB",
    "EJAA",
    "EJAB",
    "EJBA",
    "EJBB",
    "EJCA",
    "EJCB",
    "EJEA",
    "EJEB",
    "FJAA",
    "FJAB",
    "FJBA",
    "FJBB",
    "FJCA",
    "FJCB",
    "FJEA",
    "FJEB",
    "GJAA",
    "GJAB",
    "GJBA",
    "GJBB",
    "GJCA",
    "GJCB",
    "GJEA",
    "GJEB",
    "HJAA",
    "HJAB",
    "HJBA",
    "HJBB",
    "HJCA",
    "HJCB",
    "HJEA",
    "HJEB",
    "IJAA",
    "IJAB",
    "IJBA",
    "IJBB",
    "IJCA",
    "IJCB",
    "IJEA",
    "IJEB",
    "JJAA",
    "JJAB",
    "JJBA",
    "JJBB",
    "JJCA",
    "JJCB",
    "JJEA",
    "JJEB",
]


class ZINC(CachedDataset):
    """ZINC is a free database of commercially-available compounds for virtual screening."""

    def get_df(self) -> pl.DataFrame:
        df = pl.concat(
            (
                pl.read_csv(
                    f"http://files.docking.org/2D/{tranche[:2]}/{tranche}.txt",
                    separator="\t",
                )
                for tranche in tqdm(
                    ZINC20_TRANCHES, desc="ZINC20 Download", unit=" tranche"
                )
            ),
        )
        return df
